
#include "MFRC522.h"
#include "spi.h"
#include <avr/io.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>

#ifdef PC7
spi_select_class spi_mfrc522(&DDRC, &PORTC, PC7);

// Konstruktory
MFRC522_class::MFRC522_class(spi_class &spi): spi(spi)
{
	/*
	//Inicializace reset pinu
	DDRB  |= (1<<PB0);
	PORTB |= (1<<PB0);
	*/
} //MFRC522_class()

MFRC522_class::MFRC522_class(spi_class &spi, uint8_t _select_port, uint8_t _select_pin, uint8_t _reset_port, uint8_t _reset_pin): spi(spi)
{
	;; //TODO all there
} //MFRC522_class(uint8_t _select_port, uint8_t _select_pin, uint8_t _reset_port, uint8_t _reset_pin)

// Destruktor
MFRC522_class::~MFRC522_class()
{
} //~MFRC522_class



/////////////////////////////////////////////////////////////////////////////////////
// Zakladni interface pro komunikaci s MFRC522
/////////////////////////////////////////////////////////////////////////////////////


//Zapise bajt do registru
void MFRC522_class::reader_write_register(uint8_t _register, uint8_t _value)
{
	//Vybrani MFRC522
	spi_mfrc522.select(true);
	
	spi.write_data(_register & 0x7E);
	spi.write_data(_value);
	
	spi_mfrc522.select(false);
} //void reader_write_register(uint8_t _register, uint8_t _value)


//Zapise vybrany pocet bajtu z pole do registru
void MFRC522_class::reader_write_register(uint8_t _register,uint8_t _count, uint8_t *_values)
{
	spi_mfrc522.select(true);
	
	spi.write_data(_register & 0x7E);
	for (uint8_t index = 0; index < _count; index++)
	{
		spi.write_data(_values[index]);
	}
	
	spi_mfrc522.select(false);
} //void reader_write_register(uint8_t _register,uint8_t _count, uint8_t *_values)


//Precte bajt z registru
uint8_t MFRC522_class::reader_read_register(uint8_t _register)
{
	uint8_t return_value;
	
	spi_mfrc522.select(true);
	
	spi.write_data( 0x80 | (_register & 0x7E) );
	return_value = spi.write_and_read_data(0);
	
	spi_mfrc522.select(false);
	
	return return_value;
} //uint8_t reader_read_register(uint8_t _register)


//Precte zadny pocet bajtu z registru do pole
void MFRC522_class::reader_read_register(uint8_t _register, uint8_t _count, uint8_t *_values, uint8_t _rx_align /* = 0 */)
{
	if (!_count) return;
	
	uint8_t addres = 0x80 | (_register & 0x7E);
	uint8_t index  = 0;
	
	spi_mfrc522.select(true);
	
	_count--;
	
	spi.write_data(addres);
	
	while (index < _count)
	{
		if ((index == 0) && (_rx_align))
		{
			uint8_t mask = 0;
			
			for (uint8_t i = _rx_align; i <= 7; i++)
			{
				mask |= (1<<i);
			}
			
			uint8_t r_value = spi.write_and_read_data(addres);
			
			_values[0] = (_values[index] & ~mask) | (r_value & mask);
		}
		
		else
		{
			_values[index] = spi.write_and_read_data(addres);
		}
		
		index++;
	}
	
	_values[index] = spi.write_and_read_data(0);
	
	spi_mfrc522.select(false);	
} //void reader_read_register(uint8_t _register, uint8_t _count, uint8_t *_values, uint8_t _rx_align /* = 0 */)


//Nastavi bity dane maskou na registr _register
void MFRC522_class::reader_set_register_bit_mask(uint8_t _register, uint8_t _mask)
{
	uint8_t temp;
	
	temp = reader_read_register(_register);
	reader_write_register(_register, (temp | _mask));
} //void reader_set_register_bit_mask(uint8_t _register, uint8_t _mask)


//Zrusi bity dane maskou na registru _register
void MFRC522_class::reader_clear_register_bit_mask(uint8_t _register, uint8_t _mask)
{
	uint8_t temp;
	
	temp = reader_read_register(_register);
	reader_write_register(_register, (temp & (~_mask)));
} //void reader_clear_register_bit_mask(uint8_t _register, uint8_t _mask)


//Vypocet crc na koprocesoru MFRC522
//@return STATUS_OK pri uspechu, nebo STATUS_TIMEOUT
uint8_t MFRC522_class::reader_calculate_crc(uint8_t *_data, uint8_t _lenght, uint8_t *_result)
{
	reader_write_register(CommandReg, READER_Idle);	//Zastavi ostatni vykonavane prikazy
	
	reader_write_register(DivIrqReg, 0x04);				//Zrusi preruseni od CRCIRq
	reader_set_register_bit_mask(FIFOLevelReg, 0x80);	//Inicializace FIFO
	reader_write_register(FIFODataReg, _lenght, _data); //Odesle data do FIFO
	reader_write_register(CommandReg, READER_CalcCRC);	//Zacne pocitani CRC
	
	//Cekani na dokonceni vypoctu
	
	uint16_t i = 5000;
	uint8_t	 n;
	
	while (1)
	{
		n = reader_read_register(DivIrqReg);
		
		if (n & 0x04) break;	//POCITANI CRC HOTOVO
		if (--i == 0) return STATUS_TIMEOUT;	//Vyprsel timeout
	}
	
	reader_write_register(CommandReg, READER_Idle);
	
	//Precteni spocitaneho CRC
	_result[0] = reader_read_register(CRCResultRegL);
	_result[1] = reader_read_register(CRCResultRegH);
	
	return STATUS_OK;
} //uint8_t reader_calculate_crc(uint8_t *_data, uint8_t _lenght, uint8_t *_result)



/////////////////////////////////////////////////////////////////////////////////////
// Metody pro obsluhu MFRC522
/////////////////////////////////////////////////////////////////////////////////////


//Inicializace MFRC522
void MFRC522_class::reader_ini(void)
{
	DDRC  |= (1<<PC6);
	PORTC |= (1<<PC6);
	//PORTB |= (1<<PB0);	//Power on & hard reset
	wait_ms(50);	
	reader_reset();		//Soft reset
	
	//Nastaveni timeru MFRC522 pro potreby hlidani timeoutu
	//Frevence timeru = (13,56 MHz / (2*TPreScaler+1)
	//TPreScaler = [TPrescaler_Hi:TPrescaler_Lo]
	//TPrescaler_Hi jsou spodni 4 bity v TModeReg
	//TPrescaler_Lo je TPrescalerReg
	reader_write_register(TModeReg, 0x80);		//TAuto=1; Timer zacne automaticky pocitat po dokonceni vysilani
	reader_write_register(TPrescalerReg, 0xA9);	//Frekvence 40kHz, perioda 25�s 
	reader_write_register(TReloadRegH, 0x03);	//Reload timer 25ms pred timeoutem
	reader_write_register(TReloadRegL, 0xE8);	
	
	reader_write_register(TxASKReg, 0x40);
	reader_write_register(ModeReg, 0x3D);
	
	reader_set_antena_gain(GAIN_43dB);			//Nastaveni zisku anteny
	
	reader_antena_on();
} //void reader_ini(void)


//Vykonani soft resetu
void MFRC522_class::reader_reset(void)
{
	reader_write_register(CommandReg, READER_SoftReset);	//Vykona soft reset

	wait_ms(50);	//Radsi
	
	while (reader_read_register(CommandReg) & (1<<4))
	{
		;;
	}
} //void reader_reset(void)


//Zapne antenu, zapnuti pinu TX1 a TX2 (po resetu jsou vypnute)
void MFRC522_class::reader_antena_on(void)
{
	uint8_t _value = reader_read_register(TxControlReg);
	
	if ((_value & 0x03) != 0x03)
	{
		reader_write_register(TxControlReg, (_value | 0x03));
	}
} //void reader_antena_on(void)


//Nastaveni gainu anteny
void MFRC522_class::reader_set_antena_gain(uint8_t _gain)
{
	reader_write_register(RFCfgReg, _gain);		//Nastaveni zisku anteny
} //void reader_set_antena_gain(_gain)


/////////////////////////////////////////////////////////////////////////////////////
// Metody pro komunikaci s rfid kartami
/////////////////////////////////////////////////////////////////////////////////////


//Odesle data na kartu a prijme odpoved
//CRC se zvaliduje jen v pripade specifikovani _back_data, _back_lenght
//@return STATUS_OK,
uint8_t MFRC522_class::reader_transmit_and_read_data(uint8_t *_send_data, uint8_t _send_lenght, uint8_t *_back_data, uint8_t *_back_lenght, uint8_t *_valid_bits /* = NULL */, uint8_t _rx_align /* = 0 */, bool _check_crc /* = false */)
{
	uint8_t wait_irq = 0x30;	//RxIRq and IdleIRq
	
	return reader_communicate_with_card(READER_Transceive, wait_irq, _send_data, _send_lenght, _back_data, _back_lenght, _valid_bits, _rx_align, _check_crc);
} //uint8_t reader_transmit_and_read_data(uint8_t *_send_data, uint8_t _send_lenght, uint8_t *_back_data, uint8_t *_back_lenght, uint8_t *_valid_bits /* = NULL */, uint8_t _rx_align /* = 0 */, bool _check_crc /* = false */)


//Odesle data do MCRF522 FIFO, vykona prikaz, pocka na dokonceni a precte data nazpatek z FIFO
//CRC se zvaliduje jen v pripade specifikovani _back_data, _back_lenght
//@return STATUS_OK, STATUS_TIMEOUT, STATUS_false_ROOM, STATUS_COLLISION, STATUS_MIFARE_NACK, STATUS_CRC_WRONG
uint8_t MFRC522_class::reader_communicate_with_card(uint8_t _command, uint8_t _wait_irq, uint8_t *_send_data, uint8_t _send_lenght, uint8_t *_back_data /* = NULL */, uint8_t *_back_lenght /* = NULL */, uint8_t *_valid_bits /* = NULL */, uint8_t _rx_align /* = 0 */, bool _check_crc /* = false */)
{
	uint8_t n;
	uint8_t _validBits;
	uint16_t i;
	
	//Priprava dat pro BitFramingReg
	uint8_t tx_last_bits = _valid_bits ? *_valid_bits : 0;
	uint8_t bit_framing = (_rx_align << 4) + tx_last_bits;	//RxAlign = BitFramingReg[6..4]. TxLastBits = BitFramingReg[2..0]
	
	reader_write_register(CommandReg, READER_Idle);		//Zrusi vykonavani vsech prikazu
	reader_write_register(ComIrqReg, 0x7F);				//Vynuluje vsech 7 bitu preruseni
	reader_set_register_bit_mask(FIFOLevelReg, 0x80);	//inicializace FIFO
	reader_write_register(FIFODataReg, _send_lenght, _send_data);	//Zapise _send_data do FIFO
	reader_write_register(BitFramingReg, bit_framing);	//Vyrovnani bitu
	reader_write_register(CommandReg, _command);		//Odesle a vykona _commad
	
	if (_command == READER_Transceive)
	{
		reader_set_register_bit_mask(BitFramingReg, 0x80);	//Odesli data z FIFO na kartu
	}
	
	//Cekani na dokonceni prikazu
	//1 smycka while trv�  17.86us (reader_ini())
	i = 2000;
	
	while (1)
	{
		n = reader_read_register(ComIrqReg);
		if (n & _wait_irq) break;	//Preruseni ohlasilo uspech
		
		if (n & 0x01) return STATUS_TIMEOUT;	//Timer interupt - nic nebylo  prijato v 25ms
		
		if (--i == 0) return STATUS_TIMEOUT;	//Ukonceni vsech pokusu po 35.7ms
	}
	
	//Zastavit, pokud se vysktla nejaka chyba
	uint8_t error_reg_value = reader_read_register(ErrorReg);	//ErrorReg[7..0] bits are: WrErr TempErr reserved BufferOvfl   CollErr CRCErr ParityErr ProtocolErr
	
	if (error_reg_value & 0x13) return STATUS_ERROR;				//BufferOvfl ParityErr ProtocolErr	
	
	//Pokud chceme prijmout data nazpet
	if (_back_data && _back_lenght)
	{
		n = reader_read_register(FIFOLevelReg);		//Pocet bajtu v FIFO
		if (n > *_back_lenght) return STATUS_NO_ROOM;	//Neni misto na ulozeni dat
		
		*_back_lenght = n;		//Pocet dat k precteni
		reader_read_register(FIFODataReg, n, _back_data, _rx_align);	//Prijem dat z FIFO
		_validBits = reader_read_register(ControlReg) & 0x07;		//RxLastBits[2:0] urcuji pocet validnich prijmutych bitu
		
		if(_valid_bits) *_valid_bits = _validBits;
	}
	
	//Navrat koliznuho stavu
	if (error_reg_value & 0x08) return STATUS_COLLISION;
	
	//Vypocitani CRC kdyz bylo nastavefalse _check_crc = true
	if (_back_data && _back_lenght && _check_crc)
	{
		if (*_back_lenght == 1 && _validBits == 4) return STATUS_MIFARE_NACK;	//MIFARE classic NAK neni v poradku
		
		if (_send_lenght == 2 || _validBits != 0) return STATUS_CRC_WRONG;		//Nemame dostatek dat pro vypocet CRC
		
		uint8_t control_buffer[2];
		
		n = reader_calculate_crc(&_back_data[0], *_back_lenght - 2, &control_buffer[0]);
		
		if (n != STATUS_OK) return n;
		
		if ( (_back_data[*_back_lenght - 2] != control_buffer[0]) || (_back_data[*_back_lenght - 1] != control_buffer[1]) ) return STATUS_CRC_WRONG;		
	}
	
	return STATUS_OK;	
} //uint8_t reader_communicate_with_card(uint8_t _command, uint8_t _wait_irq, uint8_t *_send_data, uint8_t _send_lenght, uint8_t *_back_data /* = NULL */, uint8_t *_back_lenght /* = NULL */, uint8_t *_valid_bits /* = NULL */, uint8_t _rx_align /* = 0 */, bool _check_crc /* = false */)


//Posle pozadavek typu A. Invites PICCs in state IDLE to go to READY and prepare for anticollision or selection. 7 bit frame.
//Dve karty v dosahu muzou zp�sobovat STATUS_TIMEOUT
//@return STATUS_OK, nebo STATUS_??? pri chybe
uint8_t MFRC522_class::card_request(uint8_t *_buffer_ATQA, uint8_t *_buffer_size)
{
	return card_REQA_or_WUPA(CARD_CMD_REQA, _buffer_ATQA, _buffer_size);
} //uint8_t card_request(uint8_t *_buffer_ATQA, uint8_t *_buffer_size)


//Posle prikaz na probuzeni karty
//Dve karty v dosahu muzou zp�sobovat STATUS_TIMEOUT
//@return STATUS_OK, nebo STATUS_??? pri chybe
uint8_t MFRC522_class::card_wake_up(uint8_t *_buffer_ATQA, uint8_t *_buffer_size)
{
	return card_REQA_or_WUPA(CARD_CMD_WUPA, _buffer_ATQA, _buffer_size);
} //uint8_t card_wake_up(uint8_t *_buffer_ATQA, uint8_t *_buffer_size)


//Posle REQA nebo WUPA prikaz
//Dve karty v dosahu muzou zp�sobovat STATUS_TIMEOUT
//@return STATUS_OK, nebo STATUS_??? pri chybe
uint8_t MFRC522_class::card_REQA_or_WUPA(uint8_t _command, uint8_t *_buffer_ATQA, uint8_t *_buffer_size)
{
	uint8_t valid_bits, status;
	
	if (_buffer_ATQA == NULL || *_buffer_size < 2)	return STATUS_NO_ROOM;	//Potrebujem mit alespon 2 bity na prijem odpovedi
	
	reader_clear_register_bit_mask(CollReg, 0x80);
	valid_bits = 7; 
	status = reader_transmit_and_read_data(&_command, 1, _buffer_ATQA, _buffer_size, &valid_bits);
	
	if (status !=  STATUS_OK) return status;
	
	if (*_buffer_size != 2 || valid_bits != 0) return STATUS_ERROR;		//ATQA musi byt presne 16 bitu
	
	return STATUS_OK;
} //uint8_t card_REQA_or_WUPA(uint8_t _command, uint8_t *_buffer_ATQA, uint8_t *_buffer_size)


//Odesle SELECT/ANTICOLLISION prikaz pro vyber defifalsevane karty
//Pred odeslanim musi byt karta v ready rezimu - card_wake_up()
//Ostatni karty se prepfalseu do rezimu IDLE/HALT
/************************************************************************/
/*The UID size and value of the chosen PICC is returned in *uid along with the SAK.
*
* A PICC UID consists of 4, 7 or 10 bytes.
* Only 4 bytes can be specified in a SELECT command, so for the longer UIDs two or three iterations are used:
* 		UID size	Number of UID bytes		Cascade levels		Example of PICC
* 		========	===================		==============		===============
* 		single				 4						1				MIFARE Classic
* 		double				 7						2				MIFARE Ultralight
* 		triple				10						3				falset currently in use?
*
* @return STATUS_OK on success, STATUS_??? otherwise.
*/                                                                    
/************************************************************************/
uint8_t MFRC522_class::card_select(UID *uid, uint8_t _valid_bits /* = 0 */)
{
	bool uid_complete, select_done, use_cascade_tag;
	uint8_t cascade_level = 1,
			result, count, index,
			uid_index,					//Prvni index c uid->uid_data[] pro pouzivany cascade_level
			current_level_kfalsewn_bits;	//Pocet bitu soucasneho cascade levelu
	
	uint8_t buffer[9];					//SELECT/ANTICOLLISION prikaz + 2 bajty CRC
	uint8_t buffer_used;				//Pocet bajtu v buffer[] pro odeslani do FIFO
	uint8_t rx_align;					//Pozice prvniho prijateho bitu. Pro BitFramingReg
	uint8_t tx_last_bits;				//Pocet bitu v naposled prijate zprave. Pro BitFramingReg
	uint8_t *response_buffer;
	uint8_t response_lenght;
	
	// Description of buffer structure:
	// 		Byte 0: SEL 				Indicates the Cascade Level: PICC_CMD_SEL_CL1, PICC_CMD_SEL_CL2 or PICC_CMD_SEL_CL3
	// 		Byte 1: NVB					Number of Valid Bits (in complete command, falset just the UID): High nibble: complete bytes, Low nibble: Extra bits.
	// 		Byte 2: UID-data or CT		See explanation below. CT means Cascade Tag.
	// 		Byte 3: UID-data
	// 		Byte 4: UID-data
	// 		Byte 5: UID-data
	// 		Byte 6: BCC					Block Check Character - XOR of bytes 2-5
	//		Byte 7: CRC_A
	//		Byte 8: CRC_A
	// The BCC and CRC_A is only transmitted if we kfalsew all the UID bits of the current Cascade Level.
	//
	// Description of bytes 2-5: (Section 6.5.4 of the ISO/IEC 14443-3 draft: UID contents and cascade levels)
	//		UID size	Cascade level	Byte2	Byte3	Byte4	Byte5
	//		========	=============	=====	=====	=====	=====
	//		 4 bytes		1			uid0	uid1	uid2	uid3
	//		 7 bytes		1			CT		uid0	uid1	uid2
	//						2			uid3	uid4	uid5	uid6
	//		10 bytes		1			CT		uid0	uid1	uid2
	//						2			CT		uid3	uid4	uid5
	//						3			uid6	uid7	uid8	uid9
	
	//Kontrola zdraveho rozumu ;)	
	if (_valid_bits > 80) return STATUS_INVALID;
	
	//Priprava MFRC522
	reader_clear_register_bit_mask(CollReg, 0x80);	//ValuesAfterColl=1 => Bits received after collision are cleared. 
	
	//Opakovani cascade level smycky do precteni celeho UID
	uid_complete = false;
	
	while (! uid_complete)
	{
		//Nastaveni cascade level v SEL bajtu, zjisteny potreby pouziti cascade tag ve 2 bajtu
		switch (cascade_level)
		{
			case 1:	buffer[0] = CARD_CMD_SEL_CL1;
					uid_index = 0;
					use_cascade_tag = _valid_bits && uid->size > 4;
				break;
				
			case 2:	buffer[0] = CARD_CMD_SEL_CL2;
					uid_index = 3;
					use_cascade_tag = _valid_bits && uid->size > 7;
				break;	
				
			case 3: buffer[0] = CARD_CMD_SEL_CL3;
					uid_index = 6;
					use_cascade_tag = false;
				break;
				
			default: return STATUS_INTERNAL_ERROR;
				break;
		}	
		
		//Kolik je UID bitu v tomto cascade levelu
		current_level_kfalsewn_bits = _valid_bits - (8 * uid_index);
		if (current_level_kfalsewn_bits < 0) current_level_kfalsewn_bits = 0;
		
		//Zkopirovani znamych uid->uid_data[] do buffer[]
		index = 2;
		
		if (use_cascade_tag) buffer[index++] = CARD_CMD_CT;
		
		uint8_t bytes_to_copy = current_level_kfalsewn_bits / 8 + (current_level_kfalsewn_bits % 8 ? 1 : 0);
		
		if (bytes_to_copy)
		{
			uint8_t max_bytes = use_cascade_tag ? 3 : 4;
			
			if (bytes_to_copy > max_bytes) bytes_to_copy = max_bytes;
			
			for (count = 0; count < bytes_to_copy; count++)
			{
				buffer[index++] = uid->uid_data[uid_index + count];
			}
		}
		
		//falsew that the data has been copied we need to include the 8 bits in CT in currentLevelKfalsewnBits
		if (use_cascade_tag) current_level_kfalsewn_bits += 8;
		
		// Repeat anti collision loop until we can transmit all UID bits + BCC and receive a SAK - max 32 iterations.
		select_done = false;
		
		while (! select_done)
		{
			//Kolik bitu je potreba odeslat a prijmout
			if (current_level_kfalsewn_bits >= 32)	//All UID bits in this Cascade Level are kfalsewn. This is a SELECT.
			{
				buffer[1] = 0x70;	//NVB - pocet platnych bitu - 7
				
				//Calulate BCC - Block Check Character
				buffer[6] = buffer[2] ^ buffer[3] ^ buffer[4] ^ buffer[5];
				
				//Vypocet CRC
				result = reader_calculate_crc(buffer, 7, &buffer[7]);
				
				if (result != STATUS_OK) return result;
				
				tx_last_bits = 0;	//0 => vsech 8 bitu je v poradku
				buffer_used = 9;
				
				//Store response in the last 3 bytes of buffer (BCC and CRC_A - falset needed after tx)
				response_buffer = &buffer[6];
				response_lenght = 3;
			}
			
			else	//This is an ANTICOLLISION.
			{
				tx_last_bits = current_level_kfalsewn_bits % 8;
				count = current_level_kfalsewn_bits / 8;		//Celkovy pocet bajtu UID
				index =  2 + count;							//Number of whole bytes: SEL + NVB + UIDs
				buffer[1] = (index << 4) + tx_last_bits;
				buffer_used = index + (tx_last_bits ? 1 : 0);
				
				//Ulozeni odpovedi v nepouzite casti bufferu
				response_buffer = &buffer[index];
				response_lenght = sizeof(buffer) - index; 
			}
			
			//Bit adjustments
			rx_align = tx_last_bits;	//Having a seperate variable is overkill. But it makes the next line easier to read.
			reader_write_register(BitFramingReg, (rx_align<<4) + tx_last_bits);	//RxAlign = BitFramingReg[6..4]. TxLastBits = BitFramingReg[2..0]
			
			//Odeslani buffer[] a prijmuti odpovedi
			result = reader_transmit_and_read_data(buffer, buffer_used, response_buffer, &response_lenght, &tx_last_bits, rx_align);
			
			if (result == STATUS_COLLISION)	//Vice karet v dosahu = kolize
			{
				result = reader_read_register(CollReg);
				
				if (result & 0x20) return STATUS_COLLISION;	//Without a valid collision position we canfalset continue
			
				uint8_t collisionn_position = result & 0x1F;
			
				if (collisionn_position == 0) collisionn_position = 32;
			
				if (collisionn_position <= current_level_kfalsewn_bits) return STATUS_INTERNAL_ERROR;	//false progress - should falset happen
			
				//Vybrani karty s nastavenym bitem
				current_level_kfalsewn_bits = collisionn_position;
			
				count = (current_level_kfalsewn_bits - 1) % 8;	//Bit pro modifikaci
				index = 1 + (current_level_kfalsewn_bits / 8) + (count ? 1 : 0); //Prvni bajt je index 0
				buffer[index] |= (1<<count);
			}
			
			else if (result != STATUS_OK) return result;
			
			else  //STATUS_OK
			{
				if (current_level_kfalsewn_bits >= 32)	select_done = true;	//SELECT
				
				else current_level_kfalsewn_bits = 32;						//ANTICOLLISION
																		//Run loop again to do the SELECT.
			}			
		} //while (! select_done)
		
		
		//We do falset check the CBB - it was constructed by us above.
		
		//Copy the found UID bytes from buffer[] to uid->uid_data[]
		index = (buffer[2] == CARD_CMD_CT) ? 3 : 2;		//Zdrojovy index v buffer[]
		bytes_to_copy = (buffer[2] == CARD_CMD_CT) ? 3 : 4;
		
		for (count = 0; count < bytes_to_copy; count++)
		{
			uid->uid_data[uid_index + count] = buffer[index++];
		}
		
		//Kontrola odpovedi SAK (Select Ackfalsewledge)
		if (response_lenght != 3 || tx_last_bits != 0) return STATUS_ERROR;		//SAK je vzdy 24 bitu (1 bajt + CRC)
		
		//Kontrola CRC, do our own calculation and store the control in buffer[2..3] - those bytes are falset needed anymore.
		result = reader_calculate_crc(response_buffer, 1, &buffer[2]);
		
		if (result != STATUS_OK) return result;
		
		if ( (buffer[2] != response_buffer[1]) || (buffer[3] != response_buffer[2])) return STATUS_CRC_WRONG;
		
		if (response_buffer[0] & 0x04) cascade_level++;		//Nasteven cascade level, UID jeste neni kompletni
	
		else
		{
			uid_complete = true;
			uid->sak = response_buffer[0];
		}		
	} //while (! uid_complete)
	
	
	//Nastaveni spravneho uid->size
	uid->size = 3 * cascade_level +1;
	
	return STATUS_OK;			
} // uint8_t card_select(UID *uid, uint8_t _valid_bits /* = 0 */)


//Prepne kartu ze stavu active do stavu halt
//@return STATUS_OK, nebo STATUS_??? pro chybu
uint8_t MFRC522_class::card_halt(void)
{
	uint8_t result;
	uint8_t buffer[4];
	
	//Naplneni buffer[prikazy]
	buffer[0] = CARD_CMD_HLTA;
	buffer[1] = 0;
	
	//Vypocet CRC
	result = reader_calculate_crc(buffer, 2, &buffer[2]);
	
	if (result != STATUS_OK) return result;
	
	//Odeslani prikazu
	// The standard says:
	//		If the PICC responds with any modulation during a period of 1 ms after the end of the frame containing the
	//		HLTA command, this response shall be interpreted as 'falset ackfalsewledge'.
	// We interpret that this way: Only STATUS_TIMEOUT is an success.
	
	result = reader_transmit_and_read_data(buffer, sizeof(buffer), NULL, 0);
	
	if (result == STATUS_TIMEOUT) return STATUS_OK;	//falsermalni prubeh
	if (result == STATUS_OK) return STATUS_ERROR;	//Trocha ironie ;)
	
	return result;	
} //uint8_t card_halt(void)



/////////////////////////////////////////////////////////////////////////////////////
// Metody pro komunikaci s MIFARE kartami
/////////////////////////////////////////////////////////////////////////////////////


/**
 * Executes the MFRC522 MFAuthent command.
 * This command manages MIFARE authentication to enable a secure communication to any MIFARE Mini, MIFARE 1K and MIFARE 4K card.
 * The authentication is described in the MFRC522 datasheet section 10.3.1.9 and http://www.nxp.com/documents/data_sheet/MF1S503x.pdf section 10.1.
 * For use with MIFARE Classic PICCs.
 * The PICC must be selected - ie in state ACTIVE(*) - before calling this function.
 * Remember to call PCD_StopCrypto1() after communicating with the authenticated PICC - otherwise false new communications can start.
 * 
 * All keys are set to FFFFFFFFFFFFh at chip delivery.
 * 
 * @return STATUS_OK on success, STATUS_??? otherwise. Probably STATUS_TIMEOUT if you supply the wrong key.
 */
uint8_t MFRC522_class::reader_authenticate(uint8_t _command, uint8_t _block_addr, mifare_key *key, UID *uid)
{
	uint8_t wait_irq = 0x10;	//IdleIRq
	
	//Prikazovy buffer
	uint8_t send_data[12];
	send_data[0] = _command;
	send_data[1] = _block_addr;
	
	for (uint8_t i = 0; i < MF_KEY_SIZE; i++)
	{
		send_data[2+i] = key->key_data[i];
	}
	
	for (uint8_t i = 0; i < 4; i++)
	{
		send_data[8+i] = uid->uid_data[i];
	}
	
	//Start authentifikace
	
	return reader_communicate_with_card(READER_MFAuthent, wait_irq, &send_data[0], sizeof(send_data));
} //uint8_t reader_authenticate(uint8_t _command, uint8_t _block_addr, mifare_key *key, UID *uid)


//Ukonci authentifikaci s kartou
//Dokud to neprobehne, neni mozna komunikace s dalsi kartou!!
void MFRC522_class::reader_stop_crypto(void)
{
	reader_clear_register_bit_mask(Status2Reg, 0x08);	// Status2Reg[7..0] bits are: TempSensClear I2CForceHS reserved reserved   MFCrypto1On ModemState[2:0]
} //void reader_stop_crypto(void)


//Precte 16 bajtu + 2 bajty CRC z aktivni karty
/**
 * For MIFARE Classic the sector containing the block must be authenticated before calling this function.
 * 
 * For MIFARE Ultralight only addresses 00h to 0Fh are decoded.
 * The MF0ICU1 returns a NAK for higher addresses.
 * The MF0ICU1 responds to the READ command by sending 16 bytes starting from the page address defined by the command argument.
 * For example; if blockAddr is 03h then pages 03h, 04h, 05h, 06h are returned.
 * A roll-back is implemented: If blockAddr is 0Eh, then the contents of pages 0Eh, 0Fh, 00h and 01h are returned.
 * 
 * The buffer must be at least 18 bytes because a CRC_A is also returned.
 * Checks the CRC_A before returning STATUS_OK.
 * 
 * @return STATUS_OK on success, STATUS_??? otherwise.
 */
uint8_t MFRC522_class::mifare_read(uint8_t _block_addr, uint8_t *_buffer, uint8_t *_buffer_size)
{
	uint8_t result;
	
	if (_buffer == NULL || *_buffer_size < 18) return STATUS_NO_ROOM;
	
	//Prikazovy buffer
	_buffer[0] = CARD_CMD_MF_READ;
	_buffer[1] = _block_addr;
	
	//Vypocet CRC
	result = reader_calculate_crc(_buffer, 2, &_buffer[2]);
	
	if (result != STATUS_OK) return result;
	
	//Odeslani _buffer[], prijmuti odpovedi a kontrola CRC
	return reader_transmit_and_read_data(_buffer, 4, _buffer, _buffer_size, NULL, 0, true);
} //uint8_t mifare_read(uint8_t _block_addr, uint8_t *_buffer, uint8_t *_buffer_size)


//Zapise 16 bajtu na aktivni kartu
/**
 * For MIFARE Classic the sector containing the block must be authenticated before calling this function.
 * 
 * For MIFARE Ultralight the opretaion is called "COMPATIBILITY WRITE".
 * Even though 16 bytes are transferred to the Ultralight PICC, only the least significant 4 bytes (bytes 0 to 3)
 * are written to the specified address. It is recommended to set the remaining bytes 04h to 0Fh to all logic 0.
 * * 
 * @return STATUS_OK on success, STATUS_??? otherwise.
 */
uint8_t MFRC522_class::mifare_write(uint8_t _block_addr, uint8_t *_buffer, uint8_t _buffer_size)
{
	uint8_t result;
	
	if (_buffer == NULL || _buffer_size < 16) return STATUS_INVALID;
	
	// Mifare Classic protocol requires two communications to perform a write.
	// Step 1: Tell the PICC we want to write to block blockAddr.
	uint8_t cmd_buffer[2];
	cmd_buffer[0] = CARD_CMD_MF_WRITE;
	cmd_buffer[1] = _block_addr;
	
	result = reader_mifare_transmit_and_receive(cmd_buffer, 2);	//Adds CRC_A and checks that the response is MF_ACK.
	
	if (result != STATUS_OK) return result;
	
	// Step 2: Transfer the data
	result = reader_mifare_transmit_and_receive(_buffer, _buffer_size);	//Adds CRC_A and checks that the response is MF_ACK.
	
	if (result != STATUS_OK) return result;
	
	return STATUS_OK;
} //uint8_t mifare_write(uint8_t _block_addr, uint8_t *_buffer, uint8_t _buffer_size)


//Zapise 4 bajty do aktivni MIFARE Ultaluight karty
//@return STATUS_OK, nebo STATUS_??? pro chybu
uint8_t MFRC522_class::mifare_ultralight_write(uint8_t _page, uint8_t *_buffer, uint8_t _buffer_size)
{
	uint8_t result;
	
	if (_buffer == NULL ||_buffer_size < 4) return STATUS_INVALID;
	
	//Prikazovy buffer
	uint8_t cmd_buffer[6];
	cmd_buffer[0] = CARD_CMD_UL_WRITE;
	cmd_buffer[1] = _page;
	
	memcpy(&cmd_buffer[2], _buffer, 4);
	
	//Zapsat data
	result = reader_mifare_transmit_and_receive(cmd_buffer, 6);	//Adds CRC_A and checks that the response is MF_ACK.
	
	if (result != STATUS_OK) return result;
		
	return STATUS_OK;
} //uint8_t mifare_ultralight_write(uint8_t _page, uint8_t *_buffer, uint8_t _buffer_size)


/**
 * MIFARE Decrement subtracts the delta from the value of the addressed block, and stores the result in a volatile memory.
 * For MIFARE Classic only. The sector containing the block must be authenticated before calling this function.
 * Only for blocks in "value block" mode, ie with access bits [C1 C2 C3] = [110] or [001].
 * Use MIFARE_Transfer() to store the result in a block.
 * 
 * @return STATUS_OK on success, STATUS_??? otherwise.
 */
uint8_t MFRC522_class::mifare_decrement(uint8_t _block_addr, long _delta)
{
	return mifare_two_step_helper(CARD_CMD_MF_DECREMENT, _block_addr, _delta);
} //uint8_t mifare_decrement(uint8_t _block_addr, long _delta)


/**
 * MIFARE Increment adds the delta to the value of the addressed block, and stores the result in a volatile memory.
 * For MIFARE Classic only. The sector containing the block must be authenticated before calling this function.
 * Only for blocks in "value block" mode, ie with access bits [C1 C2 C3] = [110] or [001].
 * Use MIFARE_Transfer() to store the result in a block.
 * 
 * @return STATUS_OK on success, STATUS_??? otherwise.
 */
uint8_t MFRC522_class::mifare_increment(uint8_t _block_addr, long _delta)
{
	return mifare_two_step_helper(CARD_CMD_MF_INCREMENT, _block_addr, _delta);
} //uint8_t mifare_increment(uint8_t _block_addr, long _delta)


/**
 * MIFARE Restore copies the value of the addressed block into a volatile memory.
 * For MIFARE Classic only. The sector containing the block must be authenticated before calling this function.
 * Only for blocks in "value block" mode, ie with access bits [C1 C2 C3] = [110] or [001].
 * Use MIFARE_Transfer() to store the result in a block.
 * 
 * @return STATUS_OK on success, STATUS_??? otherwise.
 */
uint8_t MFRC522_class::mifare_restore(uint8_t _block_addr)
{
	// The datasheet describes Restore as a two step operation, but does falset explain what data to transfer in step 2.
	// Doing only a single step does falset work, so I chose to transfer 0L in step two.
	
	return mifare_two_step_helper(CARD_CMD_MF_RESTORE, _block_addr, 0L);
} //uint8_t mifare_restore(uint8_t _block_addr)


/**
 * Helper function for the two-step MIFARE Classic protocol operations Decrement, Increment and Restore.
 * 
 * @return STATUS_OK on success, STATUS_??? otherwise.
 */
uint8_t MFRC522_class::mifare_two_step_helper(uint8_t _command, uint8_t _block_addr, long _data)
{
	uint8_t result;
	uint8_t cmd_buffer[2];
	
	//Step 1: Tell the PICC the command and block address
	cmd_buffer[0] = _command;
	cmd_buffer[1] = _block_addr;
	
	result = reader_mifare_transmit_and_receive(cmd_buffer, 2);	//Adds CRC_A and checks that the response is MF_ACK.
	
	if (result != STATUS_OK) return result;
	
	//Step 2: Transfer the data
	result = reader_mifare_transmit_and_receive((uint8_t *)&_data, 4, true);	//Adds CRC_A and accept timeout as success.
	
	if (result != STATUS_OK) return result;
	
	return STATUS_OK;
} //uint8_t mifare_two_step_helper(uint8_t _command, uint8_t _block_addr, long _data)
 

/**
 * MIFARE Transfer writes the value stored in the volatile memory into one MIFARE Classic block.
 * For MIFARE Classic only. The sector containing the block must be authenticated before calling this function.
 * Only for blocks in "value block" mode, ie with access bits [C1 C2 C3] = [110] or [001].
 * 
 * @return STATUS_OK on success, STATUS_??? otherwise.
 */
uint8_t MFRC522_class::mifare_transfer(uint8_t _block_addr)
{
	uint8_t result;
	uint8_t cmd_buffer[2];
	
	//Tell the PICC we want to transfer the result into block blockAddr.
	cmd_buffer[0] = CARD_CMD_MF_TRANSFER;
	cmd_buffer[1] = _block_addr;
	
	result = reader_mifare_transmit_and_receive(cmd_buffer, 2);	//Adds CRC_A and checks that the response is MF_ACK.Adds CRC_A and checks that the response is MF_ACK.
	
	if (result != STATUS_OK) return result;
	
	return result;	
} //uint8_t mifare_transfer(uint8_t _block_addr)



/////////////////////////////////////////////////////////////////////////////////////
// Podpurne funkce
/////////////////////////////////////////////////////////////////////////////////////


/**
 * Wrapper for MIFARE protocol communication.
 * Adds CRC_A, executes the Transceive command and checks that the response is MF_ACK or a timeout.
 * 
 * @return STATUS_OK on success, STATUS_??? otherwise.
 */
uint8_t MFRC522_class::reader_mifare_transmit_and_receive(uint8_t *_send_data, uint8_t _send_lenght, bool _accept_timeout /* = false */)
{
	uint8_t result;
	uint8_t cmd_buffer[18];	//16 bajtu data, 2 bajty CRC
	
	if (_send_data == NULL || _send_lenght > 16) return STATUS_INVALID;
	
	// Copy sendData[] to cmdBuffer[] and add CRC_A
	memcpy(cmd_buffer, _send_data, _send_lenght);
	
	result = reader_calculate_crc(cmd_buffer, _send_lenght, &cmd_buffer[_send_lenght]);
	
	if (result != STATUS_OK) return result;
	
	_send_lenght += 2;
	
	//Transceive the data, store the reply in cmdBuffer[]
	uint8_t wait_irq = 0x30;	//RxIRq and IdleIRq
	uint8_t cmd_buffer_size = sizeof(cmd_buffer);
	uint8_t valid_bits = 0;
	
	result = reader_communicate_with_card(READER_Transceive, wait_irq, cmd_buffer, _send_lenght, cmd_buffer, &cmd_buffer_size, &valid_bits);
	
	if (_accept_timeout && result == STATUS_TIMEOUT) return STATUS_OK;
	
	if (result != STATUS_OK) return result;
	
	//The PICC must reply with a 4 bit ACK
	if (cmd_buffer_size != 1 || valid_bits != 4) return STATUS_ERROR;
	
	if (cmd_buffer[0] != MF_ACK) return STATUS_MIFARE_NACK;
	
	return STATUS_OK;	
} //uint8_t reader_mifare_transmit_and_receive(uint8_t *_send_data, uint8_t _send_lenght, bool _accept_timeout /* = false */)


//Returns a string pointer to a status code name.
const char *MFRC522_class::get_status_code_name(uint8_t _code)
{
	switch (_code)
	{
		case STATUS_OK:				return "Success."; break;
		case STATUS_ERROR:			return "Error in communication."; break;
		case STATUS_COLLISION:		return "Collission detected."; break;
		case STATUS_TIMEOUT:		return "Timeout in communication."; break;
		case STATUS_NO_ROOM:		return "A buffer is falset big efalseugh."; break;
		case STATUS_INTERNAL_ERROR:	return "Internal error in the code. Should falset happen."; break;
		case STATUS_INVALID:		return "Invalid argument."; break;
		case STATUS_CRC_WRONG:		return "The CRC_A does falset match."; break;
		case STATUS_MIFARE_NACK:	return "A MIFARE CARD responded with NAK."; break;
		default:					return "Unkfalsewn error"; break;
	}
} //const char *get_status_code_name(uint8_t _code)


//Translates the SAK (Select Ackfalsewledge) to a PICC type. 
// @return PICC_Type
uint8_t MFRC522_class::card_get_type(uint8_t _sak)
{
	if (_sak & 0x04)	//UID falset complete
	{ 
		return CARD_TYPE_NOT_COMPLETE;
	}
	
	switch (_sak)
	{
		case 0x09:	return CARD_TYPE_MIFARE_MINI;	break;
		case 0x08:	return CARD_TYPE_MIFARE_1K;		break;
		case 0x18:	return CARD_TYPE_MIFARE_4K;		break;
		case 0x00:	return CARD_TYPE_MIFARE_UL;		break;
		case 0x10:
		case 0x11:	return CARD_TYPE_MIFARE_PLUS;	break;
		case 0x01:	return CARD_TYPE_TNP3XXX;		break;
		default:	break;
	}
	
	if (_sak & 0x20) 
	{
		return CARD_TYPE_ISO_14443_4;
	}
	
	if (_sak & 0x40)
	{
		return CARD_TYPE_ISO_18092;
	}
	
	return CARD_TYPE_UNKNOWN;
} //uint8_t card_get_type(uint8_t _sak)


//Returns a string pointer to the PICC type name.
const char *MFRC522_class::card_get_type_name(uint8_t _type)
{
	switch (_type) 
	{
		case CARD_TYPE_ISO_14443_4:		return "CARD compliant with ISO/IEC 14443-4";		break;
		case CARD_TYPE_ISO_18092:		return "CARD compliant with ISO/IEC 18092 (NFC)";	break;
		case CARD_TYPE_MIFARE_MINI:		return "MIFARE Mini, 320 bytes";					break;
		case CARD_TYPE_MIFARE_1K:		return "MIFARE 1KB";								break;
		case CARD_TYPE_MIFARE_4K:		return "MIFARE 4KB";								break;
		case CARD_TYPE_MIFARE_UL:		return "MIFARE Ultralight or Ultralight C";			break;
		case CARD_TYPE_MIFARE_PLUS:		return "MIFARE Plus";								break;
		case CARD_TYPE_TNP3XXX:			return "MIFARE TNP3XXX";							break;
		case CARD_TYPE_NOT_COMPLETE:	return "SAK indicates UID is falset complete.";		break;
		case CARD_TYPE_UNKNOWN:
		default:						return "Unkfalsewn type";								break;
	}
} //const char *card_get_type_name(uint8_t _type)


//Calculates the bit pattern needed for the specified access bits. In the [C1 C2 C3] tupples C1 is MSB (=4) and C3 is LSB (=1).
void MFRC522_class::mifare_set_acces_bits(uint8_t *_acces_bit_buffer, uint8_t _g0, uint8_t _g1, uint8_t _g2, uint8_t _g3)
{
	uint8_t c1 = ((_g3 & 4) << 1) | ((_g2 & 4) << 0) | ((_g1 & 4) >> 1) | ((_g0 & 4) >> 2);
	uint8_t c2 = ((_g3 & 2) << 2) | ((_g2 & 2) << 1) | ((_g1 & 2) << 0) | ((_g0 & 2) >> 1);
	uint8_t c3 = ((_g3 & 1) << 3) | ((_g2 & 1) << 2) | ((_g1 & 1) << 1) | ((_g0 & 1) << 0);
	
	_acces_bit_buffer[0] = (~c2 & 0xF) << 4 | (~c1 & 0xF);
	_acces_bit_buffer[1] =          c1 << 4 | (~c3 & 0xF);
	_acces_bit_buffer[2] =          c3 << 4 | c2;
} //void mifare_set_acces_bits(uint8_t *_acces_bit_buffer, uint8_t _g0, uint8_t _g1, uint8_t _g2, uint8_t _g3)



/////////////////////////////////////////////////////////////////////////////////////
// Uzitecne metody
/////////////////////////////////////////////////////////////////////////////////////


//Byla detekovana karta? (odpoved na CARD_CMD_REQA)
//true/false
bool MFRC522_class::card_is_new_present(void)
{
	uint8_t buffer_ATQA[2];
	uint8_t buffer_size = sizeof(buffer_ATQA);
	
	uint8_t result = card_request(buffer_ATQA, &buffer_size);
	
	return (result == STATUS_OK || result == STATUS_COLLISION);
} //bool card_is_new_present(void)


/**
 * Simple wrapper around PICC_Select.
 * Returns true if a UID could be read.
 * Remember to call PICC_IsNewCardPresent(), PICC_RequestA() or PICC_WakeupA() first.
 * The read UID is available in the class variable uid.
 * 
 * @return bool
 */
bool MFRC522_class::card_read_serial(void)
{
	uint8_t result = card_select(&uid);
	
	return (result == STATUS_OK);
} //bool card_read_serial(void)
#endif

//(c) Martin Vitek 2013 (OK1KVK)
