#pragma once

#include <avr/io.h>
#include <stdint.h>

//TODO: ATmega16/32 má ADATE: ADC Auto Trigger Enable
#if defined (__AVR_ATmega16__) || (__AVR_ATmega16A__) || (__AVR_ATmega32__) || (__AVR_ATmega32A__)
	#define ADFR ADATE
#endif

#ifndef ADFR
    #define ADFR ADATE
#endif

class adc_class
{
	//Definice
	public:
		enum reference_voltages
		{
			AREF,						//Vnější reference připojená na AREF PIN
			AVCC,						//Vnitřní reference z napájecího napětí
			INTERNAL_REFERENCE			//Vnitřní reference 2.56V
		};
	
		enum result_alignment
		{
			RIGHT,
			LEFT
		};
	
		enum channels
		{
			ADC0,
			ADC1,
			ADC2,
			ADC3,
			ADC4,
			ADC5,
			ADC6,
			ADC7
		};
		
		enum prescalers
		{
			PRESCALER_2,
			PRESCALER_2_2,
			PRESCALER_4,
			PRESCALER_8,
			PRESCALER_16,
			PRESCALER_32,
			PRESCALER_64,
			PRESCALER_128
		};

	//functions
	public:
		//Konstruktory, destruktor
		adc_class();
		~adc_class();
		
		//Inicializace ADC
		void set_reference_voltage(reference_voltages reference_voltage);
		void set_result_aligment(result_alignment alignment);
		void set_prescaler(prescalers prescaler);
		void enable_free_running_mode();
		void disable_free_running_mode();
		
		void enable_adc();
		void enable_adc(bool status);

		/*
		#ifndef __AVR_ATmega8__
		void enable_auto_trrigger();
		void enable_auto_trrigger(bool _status);
		#endif
		*/

		void enable_interrupt();
		void enable_interrupt(bool _status);
		
		//Obsluha
		void start_conversion();
		void set_channel(channels channel);
		
		bool read_interrupt_flag();
		
		uint16_t read_adc();
		uint16_t read_adc(channels _channel);
}; //adc_class

