#pragma once
/**
 * circularbuffer.h v0.1
 * Jakub Skořepa 2015/8
 */

#ifndef CIRCULARBUFFER_H
#define CIRCULARBUFFER_H

#include <stdint.h>

template<typename T>
class CircularBuffer
{
	public:
		class Iterator {
				CircularBuffer *c;
				uint8_t p;
			public:
				Iterator(CircularBuffer* _c, uint8_t _p) : c(_c), p(_p) {}

				void operator++() {
					p++;
				}
				T operator*() {
					return c->store[p];
				}
				bool operator!=(Iterator i) {
					return p!=i.p;
				}
		};

		CircularBuffer() {
			read_p = 0;
			write_p = 0;
		}
		void operator << (T val) {
			store[write_p] = val;
			++write_p;
			if(no_freeze) freeze();
		}
		uint8_t operator[] (uint8_t pos) {
			return store[uint8_t(read_p+pos)];
		}
		void advance_readout(uint8_t offset) {
			read_p += offset;
		}
		uint8_t length() {
			if(write_p_cp >= read_p) return write_p_cp - read_p;
			return -(read_p - write_p_cp);
		}
		Iterator begin() {
			return Iterator(this, read_p);
		}
		Iterator end() {
			return Iterator(this, write_p_cp);
		}
		uint8_t find(T val) {
			uint8_t p = 0;
			for(p = 0; p < length(); p++) {
				if(store[uint8_t(read_p+p)]==val) return p;
			}
			return uint8_t(255);
		}
		void clear() {
			read_p = write_p_cp;
		}
		void freeze() {
			write_p_cp = write_p;
		}

		bool no_freeze = true;

	protected:
		uint8_t write_p_cp;
		uint8_t write_p;
		uint8_t read_p;
		T store[256];
};

#endif // CIRCULARBUFFER_H
