
#pragma once

#include <avr/io.h>

class output
{
	private:
		volatile uint8_t *ddr;
		volatile uint8_t *port;
				 uint8_t  bit;
	
	public:
		output( volatile uint8_t *ddr,
				volatile uint8_t *port,
						 uint8_t  bit )
		{
			this->ddr = ddr;
			this->port = port;
			this->bit = bit;

			*ddr |= (1<<bit);
		};
		
		void on()
        {
            *port |= (1<<bit);
        };

		void off()
        {
            *port &= ~(1<<bit);
        };

		void toggle()
        {
            *port ^= (1<<bit);
        };
};

class input
{
	private:
		volatile uint8_t *ddr;
		volatile uint8_t *port;
		volatile uint8_t *pin;
				 uint8_t  bit;
	public:
		input( volatile uint8_t *ddr,
			   volatile uint8_t *port,
			   volatile uint8_t *pin,
						uint8_t  bit )
        {
            this->ddr = ddr;
            this->port = port;
            this->pin = pin;
            this->bit = bit;

            *ddr &= ~(1<<bit);
        };

		~input()
        {
            set_pull_up(false);
        };
	
		void set_pull_up()
        {
            *port |= (1<<bit);
        };

		void set_pull_up(bool state)
        {
            if (state) *port |=  (1<<bit);
            else	   *port &= ~(1<<bit);
        };
		
		bool is_set()
        {
            return !(*pin & (1<<bit) );
        }
    ;
};


class io
{
	private:
		volatile uint8_t *ddr;
		volatile uint8_t *port;
		volatile uint8_t *pin;
				 uint8_t  bit;

	public:
		enum directions
		{
			INPUT,
			OUTPUT
		};

	public:
		io( volatile uint8_t *ddr,
			volatile uint8_t *port,
			volatile uint8_t *pin,
					 uint8_t  bit )
        {
            this->ddr = ddr;
            this->port = port;
            this->pin = pin;
            this->bit = bit;

            *ddr &= ~(1<<bit);
        };

		void set_direction(directions direction)
        {
            if (direction == OUTPUT) (*ddr |= (1<<bit));
            else *ddr &= ~(1<<bit);
        };

		void on()
        {
            *port |= (1<<bit);
        };

		void off()
        {
            *port &= ~(1<<bit);
        };

		void toggle()
        {
            *port ^= (1<<bit);
        };

		bool is_set()
        {
            return (*pin & (1<<bit) );
        };
};
