#include "spi.h"
#include <avr/interrupt.h>


//Konstruktory
spi_class::spi_class()
{
} //spi_class()

spi_class::spi_class(operating_modes operating_mode)
{
	set_operating_mode(operating_mode);
} //spi_class(operating_modes mode)

//Destruktor
spi_class::~spi_class()
{
	disable_SPI();
} //~spi_class()


//////////////////////////////////////////////////////////////////////////
/////////////////////////////Inicializace/////////////////////////////////
//////////////////////////////////////////////////////////////////////////

void spi_class::enable_SPI()
{
	SPCR |= (1<<SPE);
} //enable_SPI()

void spi_class::enable_SPI(bool status)
{
	if (status) SPCR |= (1<<SPE);
	else SPCR &= ~(1<<SPE);
} //enable_SPI(bool status)

void spi_class::disable_SPI()
{
	SPCR &= ~(1<<SPE);
} //disable_SPI()

void spi_class::set_operating_mode(operating_modes operating_mode)
{
	if (operating_mode == MASTER)
	{
		SPI_DDR  |=  (1<<MOSI_PIN) | (1<<SCK_PIN) | (1<<SS_PIN);
		SPI_DDR  &= ~(1<<MISO_PIN);
		SPI_PORT |=  (1<<SS_PIN);

		SPCR |= (1 << MSTR);
	}

	else
	{
		SPI_DDR |=   (1<<MISO_PIN);
		SPI_DDR &= ~((1<<MOSI_PIN) | (1<<SCK_PIN) | (1<<SS_PIN));

		SPCR &= ~(1<<MSTR);
	}
} //void set_operating_mode(uint8_t mode)

void spi_class::set_data_mode(data_modes data_mode)
{
	if (data_mode == DATA_MODE_0)
	{
		set_clock_polarity(RIS_FALL);
		set_clock_phase(SAMPLE_SETUP);
	}
	
	else if (data_mode == DATA_MODE_1)
	{
		set_clock_polarity(RIS_FALL);
		set_clock_phase(SETUP_SAMPLE);
	}
	
	else if (data_mode == DATA_MODE_2)
	{
		set_clock_polarity(FALL_RIS);
		set_clock_phase(SAMPLE_SETUP);
	}
		
	else if (data_mode == DATA_MODE_3)
	{
		set_clock_polarity(FALL_RIS);
		set_clock_phase(SETUP_SAMPLE);
	}
			
	else
	{
		set_clock_polarity(RIS_FALL);
		set_clock_phase(SAMPLE_SETUP);
	}
} //void set_data_mode(data_modes mode)

void spi_class::set_data_order(data_order order)
{
	if (order == LSB) SPCR |=  (1<<DORD);
	else			  SPCR &= ~(1<<DORD);
} //void set_data_order(data_order data_order)

void spi_class::set_clock_polarity(clock_polarity polarity)
{
	if (polarity == FALL_RIS) SPCR |=  (1<<CPOL);
	else 					  SPCR &= ~(1<<CPOL);
} //void set_clock_polarity(clock_polarity polarity)

void spi_class::set_clock_phase(clock_phase phase)
{
	if (phase == SETUP_SAMPLE) SPCR |=  (1<<CPHA);
	else 					   SPCR &= ~(1<<CPHA);
} //void set_clock_phase(clock_phase phase)

void spi_class::set_clock_rate(clock_rates rate)
{
	SPCR &= ~((1<<SPR1) | (1<<SPR0));
	SPSR &= ~(1<<SPI2X);

	SPCR |= (rate & 0b00000011);
	SPSR |= (rate & 0b00000100);
} //void set_clock_rate(clock_rates rate)

void spi_class::enable_interrupt()
{
	SPCR |= (1<<SPIE);
} //void enable_interrupt()

void spi_class::enable_interrupt(bool status)
{
	if (status) SPCR |=  (1<<SPIE);
	else 		SPCR &= ~(1<<SPIE);
} //void enable_interrupt(bool status)


//////////////////////////////////////////////////////////////////////////
////////////////////////////////Obsluha///////////////////////////////////
//////////////////////////////////////////////////////////////////////////

void spi_class::select(bool state)
{
	if (state) SPI_PORT &= ~(1<<SS_PIN);
	else	   SPI_PORT |=  (1<<SS_PIN);
} //void select(bool state)


void spi_class::write_data(uint8_t data)
{
	SPDR = data;
	
	while(!(SPSR & (1<<SPIF))) ;
	
	uint8_t dummy = SPDR;
} //void write_data(uint8_t data)

uint8_t spi_class::read_data()
{
	SPDR = 0xff;
	
	while(!(SPSR & (1<<SPIF))) ;
	
	return SPDR;
} //uint8_t read_data()

uint8_t spi_class::write_and_read_data(uint8_t data)
{
	SPDR = data;
	
	while(!(SPSR & (1<<SPIF))) ;
	
	return SPDR;
} //uint8_t write_and_read_data(uint8_t data)



//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

//Konstruktor
spi_select_class::spi_select_class(volatile uint8_t *ddr, volatile uint8_t *port, uint8_t pin)
{
	this->ddr  = ddr;
	this->port = port;
	this->pin  = pin;
	
	*ddr  |= (1<< pin);
	*port |= (1<< pin);
}

//Destruktor
spi_select_class::~spi_select_class()
{
}

void spi_select_class::select(bool status)
{	
	if (status) *port &= ~(1<< pin);
	else 		*port |=  (1<< pin);
} //void select(bool status)


//(c) Martin Vitek 2015 (OK1KVK)