#pragma once

#include <avr/io.h>
#include <stdint.h>


#if defined(__AVR_ATmega8__)
	#define SPI_DDR		DDRB
	#define SPI_PORT	PORTB
	#define MOSI_PIN	PB3
	#define MISO_PIN	PB4
	#define SCK_PIN		PB5
	#define SS_PIN		PB2

#elif defined(__AVR_ATmega16__) || defined(__AVR_ATmega32__) || defined(__AVR_ATmega32A__)
	#define SPI_DDR		DDRB
	#define SPI_PORT	PORTB
	#define MOSI_PIN	PB5
	#define MISO_PIN	PB6
	#define SCK_PIN		PB7
	#define SS_PIN		PB4

#elif defined(__AVR_ATmega128__) || (__AVR_ATmega1281__)
	#define SPI_DDR    	DDRB
	#define SPI_PORT	PORTB
	#define MOSI_PIN	PB2
	#define MISO_PIN	PB3
	#define SCK_PIN		PB1
	#define SS_PIN		PB0

#else
	#error "This lib don't support this device"
#endif


class spi_class
{
	//Definice
	public:
		enum operating_modes
		{
			MASTER,
			SLAVE
		};
		
		enum data_modes
		{
			DATA_MODE_0	= 0,
			DATA_MODE_1	= 1,
			DATA_MODE_2	= 2,
			DATA_MODE_3	= 3
		};
		
		enum data_order
		{
			MSB	= 1,
			LSB	= 0
		};
		
		enum clock_polarity	
		{
			RIS_FALL = 0,
			FALL_RIS = 1
		};
		
		enum clock_phase
		{
			SAMPLE_SETUP = 0,
			SETUP_SAMPLE = 1
		};
		
		enum clock_rates
		{
			F_OSC_2		= 4,
			F_OSC_4		= 0,
			F_OSC_8		= 5,
			F_OSC_16	= 1,
			F_OSC_32	= 6,
			F_OSC_64	= 2,
			F_OSC_64_2  = 7,
			F_OSC_128	= 3
		};	

	//Metody
	public:
		//Konstruktory, destruktor
		spi_class();
		spi_class(operating_modes mode);
		~spi_class();
		
		//Inicializace
		void enable_SPI();
		void enable_SPI(bool status);
		void disable_SPI();
		void set_operating_mode(operating_modes operating_mode);
		void set_data_mode(data_modes data_mode);
		void set_data_order(data_order order);
		void set_clock_polarity(clock_polarity polarity);
		void set_clock_phase(clock_phase phase);
		void set_clock_rate(clock_rates rate);
		void enable_interrupt();
		void enable_interrupt(bool status);
		
		//Obsluha
		void select(bool state);
		
		void 	write_data(uint8_t data);
		uint8_t read_data();
		uint8_t write_and_read_data(uint8_t data);
}; //spi_class

class spi_select_class
{	
	private:
		volatile uint8_t *ddr;
		volatile uint8_t *port;
				 uint8_t  pin;
	
	public:
		spi_select_class(volatile uint8_t *ddr, volatile uint8_t *port, uint8_t pin);
		~spi_select_class();
		
		void select(bool status);
}; //spi_select_class


//(c) Martin Vítek 2015 (OK1KVK)