
#include "usart.h"

#include <avr/io.h>
#include <stdint.h>

//////////////////////////////////////////////////////////////////////////
////////////////////////Konstruktor, destruktor///////////////////////////
//////////////////////////////////////////////////////////////////////////

usart::usart(usart_regs &regs): reg(regs)
{
	flush_UDR_buffer();
}

usart::~usart()
{
	flush_UDR_buffer();
}


//////////////////////////////////////////////////////////////////////////
/////////////////////////////Inicializace/////////////////////////////////
//////////////////////////////////////////////////////////////////////////

void usart::set_operating_mode(operation_modes mode)
{
	//Nastaveni modu komunikace
	this->mode = mode;

	uint8_t mask = *reg.UCSRC_reg;
	mask = *reg.UCSRC_reg;
	mask &= ~(1<<UMSEL);
	mask |=  (1<<URSEL);
	*reg.UCSRC_reg = mask;


	if (mode == ASYNCHRONOUS_NORMAL) *reg.UCSRA_reg &= ~(1<<U2X);

	else if (mode == ASYNCHRONOUS_DOUBLE) *reg.UCSRA_reg |= (1<<U2X);

	else if (mode == SYNCHRONOUS)
	{
		*reg.UCSRC_reg |= (1<<URSEL) | (1<<UMSEL);
		*reg.UCSRA_reg &= ~(1<<U2X);
	}

	else return;
}

void usart::set_baudrate(uint32_t baudrate)
{
	uint16_t baud_value = ((F_CPU/(mode*baudrate))-1);

	*reg.UBRRH_reg = (uint8_t)(baud_value>>8);
	*reg.UBRRL_reg = (uint8_t) baud_value;
}

void usart::set_data_bits(uint8_t bits)
{
	uint8_t mask = *reg.UCSRC_reg;
	mask = *reg.UCSRC_reg;
	mask &= ~((1<<UCSZ1) | (1<<UCSZ0));
	mask |=  (1<<URSEL);
	*reg.UCSRC_reg = mask;
	*reg.UCSRB_reg &= ~(1<<UCSZ2);

	switch (bits)
	{
		case 5:
			break;

		case 6:	*reg.UCSRC_reg |= (1<<URSEL) | (1<<UCSZ0);
			break;

		case 7:	*reg.UCSRC_reg |= (1<<URSEL) | (1<<UCSZ1);
			break;

		case 8:
				*reg.UCSRC_reg |= (1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0);
			break;

		case 9:	*reg.UCSRB_reg |= (1<<UCSZ2);
				*reg.UCSRC_reg |= (1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0);
			break;

		default:
			break;
	}
}

void usart::set_stop_bits(uint8_t bits)
{
	switch (bits)
	{
		case 1:	*reg.UCSRC_reg&= ~(1<<USBS) | (1<<URSEL);
			break;

		case 2:	*reg.UCSRC_reg|= (1<<USBS) | (1<<URSEL);
			break;

		default: *reg.UCSRC_reg&= ~(1<<USBS) | (1<<URSEL);
			break;
	}
}

void usart::set_parity(parity_modes parity)
{
	switch (parity)
	{
		case PARITY_NONE: *reg.UCSRC_reg &= (1<<URSEL) | ~(1<<UPM1) | ~(1<<UPM0);
			break;

		case PARITY_EVEN: *reg.UCSRC_reg |= (1<<URSEL) |  (1<<UPM1) | ~(1<<UPM0);
			break;

		case PARITY_ODD: *reg.UCSRC_reg &= (1<<URSEL) | ~(1<<UPM1) |  (1<<UPM0);
			break;
	}
}

void usart::enable_reciver()
{
	*reg.UCSRB_reg |= (1<<RXEN);
}

void usart::enable_reciver(bool status)
{
	if (status) *reg.UCSRB_reg |=  (1<<RXEN);
	else 		*reg.UCSRB_reg &= ~(1<<RXEN);
}

void usart::enable_trasmitter()
{
	*reg.UCSRB_reg |= (1<<TXEN);
}

void usart::enable_trasmitter(bool status)
{
	if (status) *reg.UCSRB_reg |=  (1<<TXEN);
	else 		*reg.UCSRB_reg &= ~(1<<TXEN);
}

void usart::enable_UDRE_interrupt()
{
	*reg.UCSRB_reg |= (1<<UDRIE);
}

void usart::enable_UDRE_interrupt(bool status)
{
	if (status) *reg.UCSRB_reg |=  (1<<UDRIE);
	else 		*reg.UCSRB_reg &= ~(1<<UDRIE);
}

void usart::enable_TX_complete_interrupt()
{
	*reg.UCSRB_reg |= (1<<TXCIE);
}

void usart::enable_TX_complete_interrupt(bool status)
{
	if (status) *reg.UCSRB_reg |=  (1<<TXCIE);
	else 		*reg.UCSRB_reg &= ~(1<<TXCIE);
}

void usart::enable_RX_complete_interrupt()
{
	*reg.UCSRB_reg |= (1<<RXCIE);
}

void usart::enable_RX_complete_interrupt(bool status)
{
	if (status) *reg.UCSRB_reg |=  (1<<RXCIE);
	else 		*reg.UCSRB_reg &= ~(1<<RXCIE);
}

void usart::enable_MPCM()
{
	*reg.UCSRA_reg |= (1<<MPCM);
}

void usart::enable_MPCM(bool status)
{
	if (status) *reg.UCSRA_reg |=  (1<<MPCM);
	else 		*reg.UCSRA_reg &= ~(1<<MPCM);
}


//////////////////////////////////////////////////////////////////////////
//////////////////////////////Funkce//////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

/////////////////////////////Transmit/////////////////////////////////////
void usart::transmit_char(uint8_t data)
{
	while(!(*reg.UCSRA_reg & (1<<UDRE))) ;;

	*reg.UDR_reg = data;
}

void usart::transmit_string(const char *data)
{
	while (*data) transmit_char(*(data++));
}

void usart::transmit_data(const char *data, uint8_t size)
{
	uint8_t count = 0;
	while(count < size)
	{
		transmit_char(*(data++));
		count++;
	}
}

void usart::transmit_dec32(long data)
{
	unsigned long n;
	char ignoruj;

	if(data<0){
		data=-data;
		transmit_char('-');
	}

	ignoruj=1;
	for(n=1000000000; n>0; n/=10){
		if(data/n){
			transmit_char('0'+data/n);
			ignoruj=0;
			data%=n;
			}else if(!ignoruj){
			transmit_char('0');
		}
	}
	if(ignoruj) transmit_char('0');
}

void usart::transmit_hex8(uint8_t data)
{
	if(data<0xA0) transmit_char((data>>4)+'0');
	else transmit_char((data>>4)+'A'-10);

	if((data&0x0f)<0x0A) transmit_char((data&0x0f)+'0');
	else transmit_char((data&0x0f)+'A'-10);
}

void usart::transmit_hex16(uint16_t data)
{
	transmit_hex8(data>>8);
	transmit_hex8(data&0xff);
}

void usart::transmit_hex32(uint32_t data)
{
	transmit_hex8(data>>24);
	transmit_hex8(data>>16);
	transmit_hex8(data>>8);
	transmit_hex8(data&0xff);
}

void usart::transmit_new_line()
{
	transmit_string("\n\r");
} //void transmit_new_line()

void usart::transmit_space()
{
	transmit_char(' ');
} //void transmit_space()

void usart::transmit_binary8(uint8_t data)
{
	for(signed char i = 7; i > -1; i--)
	{
		transmit_dec32((data&(1<<i))>>i);
	}
} //void transmit_binary8(uint8_t data)

void usart::transmit_binary16(uint16_t data)
{
	for(signed char i = 15; i > -1; i--)
	{
		transmit_dec32((data&(1<<i))>>i);
	}
} //void transmit_binary16(uint16_t data)

void usart::transmit_binary32(uint32_t data)
{
	for(signed char i = 31; i > -1; i--)
	{
		transmit_dec32((data&(1<<i))>>i);
	}
} //void transmit_binary32(uint32_t data)

/////////////////////////////Receive//////////////////////////////////////
uint8_t usart::receive_char()
{
	while(!(*reg.UCSRA_reg & (1<<RXC))) ;;

	return *reg.UDR_reg;
}


////////////////////////////Obsluha///////////////////////////////////////
bool usart::check_frame_error()
{
	return (bool)(*reg.UCSRA_reg&(1<<FE));
}

bool usart::check_data_over_run()
{
	return (bool)(*reg.UCSRA_reg&(1<<DOR));
}

bool usart::check_parity_error()
{
	return (bool)(*reg.UCSRA_reg&(1<<UPE));
}


bool usart::check_usart_recive_complete()
{
	return (bool)(*reg.UCSRA_reg&(1<<RXC));
}

bool usart::check_usart_transmit_complete()
{
	return (bool)(*reg.UCSRA_reg&(1<<TXC));
}

bool usart::check_usart_data_register_empty()
{
	return (bool)(*reg.UCSRA_reg&(1<<UDRE));
}


void usart::flush_UDR_buffer()
{
	unsigned char dummy1, dummy2;
	while( *reg.UCSRA_reg & (1<<RXC) ) dummy1 = *reg.UDR_reg;
	dummy2 = dummy1;
	dummy1 = dummy2;
}

//(c) Martin Vitek 2015 (OK1KVK)
