#pragma once

#include <avr/io.h>
#include <stdint.h>

#ifndef UPE
    #if defined(PE)
	    #define UPE PE
    #elif defined(UPE0)
        #define UPE UPE0
    #else
        #warning "Can't define UPE"
    #endif
#endif

#ifndef URSEL
	#define URSEL 7
#endif

#ifndef UMSEL
    #define UMSEL UMSEL00
#endif

#ifndef U2X
    #define U2X U2X0
#endif

#ifndef UCSZ0
    #define UCSZ0 UCSZ00
#endif

#ifndef UCSZ1
    #define UCSZ1 UCSZ01
#endif

#ifndef UCSZ2
    #define UCSZ2 UCSZ02
#endif

#ifndef USBS
    #define USBS USBS0
#endif

#ifndef UPM0
    #define UPM0 UPM00
#endif

#ifndef UPM1
    #define UPM1 UPM01
#endif

#ifndef RXEN
    #define RXEN RXEN0
#endif

#ifndef TXEN
    #define TXEN TXEN0
#endif

#ifndef UDRIE
    #define UDRIE UDRIE0
#endif

#ifndef TXCIE
    #define TXCIE TXCIE0
#endif

#ifndef RXCIE
    #define RXCIE RXCIE0
#endif

#ifndef MPCM
    #define MPCM MPCM0
#endif

#ifndef UDRE
    #define UDRE UDRE0
#endif

#ifndef FE
    #define FE FE0
#endif

#ifndef DOR
    #define DOR DOR0
#endif

#ifndef DOR
    #define DOR DOR0
#endif

#ifndef RXC
    #define RXC RXC0
#endif

#ifndef TXC
    #define TXC TXC0
#endif

#ifndef RXEN
    #define RXEN RXEN0
#endif

#ifndef URSEL
	#define URSEL 7
#endif

struct usart_regs
{
	volatile uint8_t *UCSRA_reg;
	volatile uint8_t *UCSRB_reg;
	volatile uint8_t *UCSRC_reg;
	volatile uint8_t *UDR_reg;
	volatile uint8_t *UBRRL_reg;
	volatile uint8_t *UBRRH_reg;
};

class usart
{
	public:
		enum operation_modes
		{
			ASYNCHRONOUS_NORMAL = 16,
			ASYNCHRONOUS_DOUBLE = 8,
			SYNCHRONOUS			= 2
		};

		enum parity_modes
		{
			PARITY_NONE = 0,
			PARITY_EVEN = 2,
			PARITY_ODD  = 3
		};

	private:
		usart_regs reg;
		operation_modes mode;

	public:
		//Konstruktor, destruktor
		usart(usart_regs &regs);
		~usart();


	//USART Init
	void set_operating_mode(operation_modes mode);
	void set_baudrate(uint32_t baudrate);
	void set_data_bits(uint8_t bits);
	void set_stop_bits(uint8_t bits);
	void set_parity(parity_modes parity);
	void enable_reciver();
	void enable_reciver(bool status);
	void enable_trasmitter();
	void enable_trasmitter(bool status);
	void enable_UDRE_interrupt();	//Data Register Empty
	void enable_UDRE_interrupt(bool status);
	void enable_TX_complete_interrupt();
	void enable_TX_complete_interrupt(bool status);
	void enable_RX_complete_interrupt();
	void enable_RX_complete_interrupt(bool status);
	void enable_MPCM();				//Multi Processor Communication Mode
	void enable_MPCM(bool status);	//Datasheet page 145

	//USART functions
	void transmit_char(uint8_t data);
	void transmit_string(const char *data);
	void transmit_data(const char *data, uint8_t size);
	void transmit_dec32(long data);		//Funkce pro odeslani 32bit desitkoveho cisla
	void transmit_hex8(uint8_t data);	//Funkce pro odeslani 1 bajtoveho Hexu
	void transmit_hex16(uint16_t data);	//Funkce pro odeslani 2 bajtoveho Hexu
	void transmit_hex32(uint32_t data);	//Funkce pro odeslani 4 bajtoveho Hexu
	void transmit_binary8(uint8_t data);
	void transmit_binary16(uint16_t data);
	void transmit_binary32(uint32_t data);

	void transmit_new_line();		//Odesle \n\r
	void transmit_space();			//Odesle mezeru



	uint8_t receive_char();

	bool check_frame_error();		//Dataseet page 149
	bool check_data_over_run();		//Dataseet page 149
	bool check_parity_error();		//Dataseet page 149

	bool check_usart_recive_complete();
	bool check_usart_transmit_complete();
	bool check_usart_data_register_empty();

	void flush_UDR_buffer();
};


//////////////////////////////////////////////////////////////////////////
//////////////////Vektory preruseni RX, TX, UDRE//////////////////////////
//////////////////////////////////////////////////////////////////////////

/*
ISR(USART_RXC_vect){};
ISR(USART_TXC_vect){};
ISR(USART_UDRE_vect){};
*/


//(c) Martin Vitek 2015 (OK1KVK)
