#!/bin/bash

project="gate-servo-controller"
schema="HW/gate-servo-controller.sch"
board="HW/gate-servo-controller.brd"

# Check, if there are changes to be commited
do_sch=$(git status --porcelain | awk '/^M/ && /.sch/')
do_brd=$(git status --porcelain | awk '/^M/ && /.brd/')

# Generate schema image
if [ "$do_sch" ]; then
	eagle -C "set confirm yes;
        	  export image $project-schema.png 300;
          	  set confirm no;
          	  quit;" $schema
fi

# Generate board image
if [ "$do_brd" ]; then
	eagle -C "set confirm yes;
        	  display None;
          	  display Top Bottom Pads Vias Unrouted Dimension tPlace bPlace tName bNames Measures Document tDocu bDocu;
          	  ratsnest;
          	  export image $project-pcb.png 300;
	          set confirm no;
        	  quit;" $board
fi

# Generate schema PDF
if [ "$do_sch" ]; then
	eagle -C "set confirm yes;
        	  print -1 -mirror -caption -rotate -upsidedown -black -solid landscape paper A4 file %N-schema.pdf;
	          set confirm no;
	          quit;" $schema

	mv HW/*.pdf .
fi

git add $project-pcb.png
git add $project-schema.png
git add $project-schema.pdf
